"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require("path");

var _child_process = require("child_process");

var _babelTypes = require("babel-types");

var globalPath = process.cwd();
var compileDir = "lib";
var fileRegex = /\.(re|ml)$/;

var bsb = void 0;

try {
  bsb = require.resolve("bs-platform/bin/bsb.exe");
} catch (e) {
  bsb = "bsb";
}

var isBuckleScriptFile = function isBuckleScriptFile(path) {
  return fileRegex.test(path);
};

var traverseExpression = function traverseExpression(t, arg) {
  if (t.isStringLiteral(arg)) {
    return arg;
  }

  if (t.isBinaryExpression(arg)) {
    return traverseExpression(t, arg.left);
  }

  return null;
};

var getModuleType = function getModuleType(state) {
  return state.opts.module || "js";
};

var getNewPath = function getNewPath(path, state) {
  var sourcePath = (0, _path.dirname)(state.file.opts.filenameRelative);
  var requirePath = (0, _path.resolve)(sourcePath, path);
  var rootPath = requirePath.replace(globalPath, "");
  var newPath = (0, _path.join)(globalPath, compileDir, getModuleType(state), rootPath);

  return newPath.replace(fileRegex, ".js");
};

var compile = function compile() {
  (0, _child_process.execFileSync)(bsb, ["-make-world"], { stdio: "pipe" });
};

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      CallExpression: function CallExpression(path, state) {
        if (path.node.callee.name !== "require") return;

        var args = path.node.arguments;

        if (!args.length) return;

        var firstArg = traverseExpression(t, args[0]);

        if (!firstArg) return;

        if (!isBuckleScriptFile(firstArg.value)) return;

        var newPath = getNewPath(firstArg.value, state);

        compile();

        path.node.arguments = [newPath];
      },
      ImportDeclaration: function ImportDeclaration(path, state) {
        if (!isBuckleScriptFile(path.node.source.value)) return;

        var newPath = getNewPath(path.node.source.value, state);

        compile();

        path.node.source.value = newPath;
      }
    }
  };
};